@@ -32,27 +32,30 @@
 #include <fcntl.h>
 
 #include "buffer.h"
+#include "md5.h"
 
 ret_t
 cherokee_buffer_new  (cherokee_buffer_t **buf)
 {
-	   CHEROKEE_NEW_STRUCT(n, buffer);
+	CHEROKEE_NEW_STRUCT(n, buffer);
 
-	   n->buf  = NULL;
-	   n->size = 0;
-	   n->len  = 0;
+	n->buf  = NULL;
+	n->size = 0;
+	n->len  = 0;
 	   
-	   *buf = n;
+	*buf = n;
 	   
-	   return ret_ok;
+	return ret_ok;
 }
 
 
 ret_t
 cherokee_buffer_free (cherokee_buffer_t *buf)
 {
-	if (buf->buf)
+	if (buf->buf) {
 		free (buf->buf);
+		buf->buf = NULL;
+	}
 	
 	free (buf);
 	
@@ -63,32 +66,42 @@ cherokee_buffer_free (cherokee_buffer_t *buf)
 ret_t
 cherokee_buffer_clean (cherokee_buffer_t *buf)
 {
-	   return cherokee_buffer_make_empty (buf);
+	return cherokee_buffer_make_empty (buf);
 }
 
 
 ret_t
-cherokee_buffer_add (cherokee_buffer_t  *buf, char *txt, int size)
+cherokee_buffer_add (cherokee_buffer_t *buf, char *txt, int size)
 {	   
-	   int free = buf->size - buf->len;
+	int free = buf->size - buf->len;
 
-	   /* Get memory
-	    */
-	   if (free <= (size+1)) {
-		   buf->buf = (char *) realloc(buf->buf, buf->size + size - free + 1);
-		   return_if_fail (buf->buf, ret_nomem);
-		   
-		   buf->size += size - free + 1;
-	   }
-	   
-	   /* Copy	
-	    */
-	   memcpy (buf->buf + buf->len, txt, size);
+	if (size <= 0)
+		return ret_ok;
 
-	   buf->len += size;
-	   buf->buf[buf->len] = '\0';
+	/* Get memory
+	 */
+	if (free < (size+1)) {
+		buf->buf = (char *) realloc(buf->buf, buf->size + size - free + 1);
+		return_if_fail (buf->buf, ret_nomem);
+
+		buf->size += size - free + 1;
+	}
 	   
-	   return ret_ok;
+	/* Copy	
+	 */
+	memcpy (buf->buf + buf->len, txt, size);
+
+	buf->len += size;
+	buf->buf[buf->len] = '\0';
+	   
+	return ret_ok;
+}
+
+
+ret_t 
+cherokee_buffer_add_buffer (cherokee_buffer_t *buf, cherokee_buffer_t *buf2)
+{
+	return cherokee_buffer_add (buf, buf2->buf, buf2->len);
 }
 
 
@@ -112,53 +125,55 @@ cherokee_buffer_add_va (cherokee_buffer_t  *buf, char *format, ...)
 ret_t
 cherokee_buffer_prepend (cherokee_buffer_t  *buf, char *txt, int size)
 {
-	   int free = buf->size - buf->len;
+	int free = buf->size - buf->len;
 
-	   /* Get memory
-	    */
-	   if (free <= size) {
-		   buf->buf = (char *) realloc(buf->buf, buf->size + size - free + 1);
-		   return_if_fail (buf->buf, ret_nomem);
+	/* Get memory
+	 */
+	if (free <= size) {
+		buf->buf = (char *) realloc(buf->buf, buf->size + size - free + 1);
+		return_if_fail (buf->buf, ret_nomem);
 		   
-		   buf->size += size - free + 1;
-	   }
+		buf->size += size - free + 1;
+	}
 
-	   memmove (buf->buf+size, buf->buf, buf->len);
+	memmove (buf->buf+size, buf->buf, buf->len);
 
-	   memcpy (buf->buf, txt, size);
-	   buf->len += size;
-	   buf->buf[buf->len] = '\0';
+	memcpy (buf->buf, txt, size);
+	buf->len += size;
+	buf->buf[buf->len] = '\0';
 	   
-	   return ret_ok;
+	return ret_ok;
 }
 
 
 int
 cherokee_buffer_is_empty (cherokee_buffer_t *buf)
 {
-	   return (buf->len == 0);
+	return (buf->len == 0);
 }
 
 
 ret_t
 cherokee_buffer_make_empty (cherokee_buffer_t *buf)
 {
-	   buf->len = 0;
-	   return ret_ok;
+	buf->len = 0;
+	return ret_ok;
 }
 
 
 ret_t
 cherokee_buffer_move_to_begin (cherokee_buffer_t *buf, int pos)
 {
-	   return_if_fail (pos <= buf->size, ret_error);
+	return_if_fail (pos <= buf->size, ret_error);
 
-	   if (pos <= buf->len) {
-			 memcpy (buf->buf, buf->buf+pos, buf->size-pos);
-			 buf->len -= pos;
-	   }
+	if (pos <= 0) return ret_ok;
+
+	if (pos <= buf->len) {
+		memcpy (buf->buf, buf->buf+pos, buf->size-pos);
+		buf->len -= pos;
+	}
 	   
-	   return ret_ok;
+	return ret_ok;
 }
 
 
@@ -188,6 +203,8 @@ cherokee_buffer_decode (cherokee_buffer_t *buffer)
 	char *from;
 	char *to;
 
+	if (buffer->buf == NULL) return;
+
 	from = to = buffer->buf;
 
 	for (; *from != '\0'; ++to, ++from) {
@@ -353,3 +370,160 @@ cherokee_buffer_add_version (cherokee_buffer_t *buf, int port, cherokee_version_
 
 	return ret;
 }
+
+
+/* b64_decode_table has been copy&pasted from the code of thttpd:  
+ * Copyright <A9> 1995,1998,1999,2000,2001 by Jef Poskanzer <jef@acme.com>
+ */
+
+/* Base-64 decoding.  This represents binary data as printable ASCII
+ * characters.  Three 8-bit binary bytes are turned into four 6-bit
+ * values, like so:
+ *
+ *   [11111111]  [22222222]  [33333333]
+ *   [111111] [112222] [222233] [333333]
+ *
+ * Then the 6-bit values are represented using the characters "A-Za-z0-9+/".
+ */
+
+static int 
+b64_decode_table[256] = {
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 00-0F */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 10-1F */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,62,-1,-1,-1,63,  /* 20-2F */
+	52,53,54,55,56,57,58,59,60,61,-1,-1,-1,-1,-1,-1,  /* 30-3F */
+	-1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,  /* 40-4F */
+	15,16,17,18,19,20,21,22,23,24,25,-1,-1,-1,-1,-1,  /* 50-5F */
+	-1,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,  /* 60-6F */
+	41,42,43,44,45,46,47,48,49,50,51,-1,-1,-1,-1,-1,  /* 70-7F */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 80-8F */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 90-9F */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* A0-AF */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* B0-BF */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* C0-CF */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* D0-DF */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* E0-EF */
+	-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1   /* F0-FF */
+};
+
+
+ret_t 
+cherokee_buffer_decode_base64 (cherokee_buffer_t *buf)
+{
+	char space[128];
+	int  space_idx = 0;
+	int  i, phase  = 0;
+	int  d, prev_d = 0;
+	int  buf_pos   = 0;
+
+	space_idx = 0;
+	phase = 0;
+	for (i=0; i < buf->len; i++) {
+		d = b64_decode_table[(int) buf->buf[i]];
+		if (d != -1) {
+			switch (phase) {
+			case 0:
+				++phase;
+				break;
+			case 1:
+				space[space_idx++] = (( prev_d << 2 ) | ( ( d & 0x30 ) >> 4 ));
+				++phase;
+				break;
+			case 2:
+				space[space_idx++] = (( ( prev_d & 0xf ) << 4 ) | ( ( d & 0x3c ) >> 2 ));
+				++phase;
+				break;
+			case 3:
+				space[space_idx++] = (( ( prev_d & 0x03 ) << 6 ) | d );
+				phase = 0;
+				break;
+			}
+			prev_d = d;
+		} 
+
+		if (space_idx == 127) {
+			memcpy (buf->buf + buf_pos, space, 127);
+			buf_pos += 127;
+			space_idx = 0;
+		}
+        }
+
+	space[space_idx]='\0';
+
+	memcpy (buf->buf + buf_pos, space, space_idx+1);
+	buf->len = buf_pos + space_idx;
+	
+	return ret_ok;
+}
+
+
+
+/* Documentation: 
+ * RFC 1321, `The MD5 Message-Digest Algorithm'
+ * http://www.alobbs.com/modules.php?op=modload&name=rfc&file=index&content_file=rfc1321.php
+ */
+
+ret_t 
+cherokee_buffer_encode_md5 (cherokee_buffer_t *buf)
+{
+	int i;
+	MD5_CTX context;
+	unsigned char digest[16];
+
+	MD5Init (&context);
+	MD5Update (&context, buf->buf, buf->len);
+	MD5Final (digest, &context);
+
+	cherokee_buffer_ensure_size (buf, 34);
+	
+	for (i=0; i<16; i++) {
+		sprintf (buf->buf+(i*2), "%02x", digest[i]);
+	}
+	buf->buf[32] = '\0';
+	buf->len = 32;
+
+	return ret_ok;
+}
+
+
+/* Documentation:
+ * RFC 2617, HTTP Authentication: Basic and Digest Access Authentication
+ * http://www.alobbs.com/modules.php?op=modload&name=rfc&file=index&content_file=rfc2617.php
+ */
+
+ret_t 
+cherokee_buffer_encode_hex (cherokee_buffer_t *buf)
+{
+	unsigned int  i;
+	unsigned char j;
+	char *new_buf;
+
+	new_buf = (char *) malloc((buf->len * 2)+1);
+	if (new_buf == NULL) {
+		return ret_error;
+	}
+
+	for (i = 0; i < buf->len; i++) {
+		j = (buf->buf[i] >> 4) & 0xf;
+		if (j <= 9)
+			new_buf[i*2] = (j + '0');
+		else
+			new_buf[i*2] = (j + 'a' - 10);
+
+		j = buf->buf[i] & 0xf;
+		if (j <= 9)
+			new_buf[i*2+1] = (j + '0');
+		else
+			new_buf[i*2+1] = (j + 'a' - 10);
+	}
+
+	new_buf[buf->len*2] = '\0';
+
+	free (buf->buf);
+
+	buf->len *= 2;
+	buf->size = buf->len + 1;
+	buf->buf  = new_buf;
+
+	return ret_ok;
+}
