@@ -50,6 +50,7 @@ ret_t cherokee_buffer_clean         (cherokee_buffer_t  *buf);
 ret_t cherokee_buffer_add           (cherokee_buffer_t  *buf, char *txt, int size);
 ret_t cherokee_buffer_add_va        (cherokee_buffer_t  *buf, char *format, ...);
 ret_t cherokee_buffer_read_file     (cherokee_buffer_t  *buf, char *filename);
+ret_t cherokee_buffer_add_buffer    (cherokee_buffer_t  *buf, cherokee_buffer_t *buf2);
 
 ret_t cherokee_buffer_prepend       (cherokee_buffer_t  *buf, char *txt, int size);
 
@@ -61,10 +62,13 @@ ret_t cherokee_buffer_ensure_size   (cherokee_buffer_t  *buf, int size);
 int   cherokee_buffer_is_empty      (cherokee_buffer_t  *buf);
 ret_t cherokee_buffer_make_empty    (cherokee_buffer_t  *buf);
 
-void  cherokee_buffer_decode        (cherokee_buffer_t  *buf);
 crc_t cherokee_buffer_crc32         (cherokee_buffer_t  *buf);
-ret_t cherokee_buffer_add_version   (cherokee_buffer_t  *buf, int port, cherokee_version_t ver);
+void  cherokee_buffer_decode        (cherokee_buffer_t  *buf);
+ret_t cherokee_buffer_decode_base64 (cherokee_buffer_t  *buf);
+ret_t cherokee_buffer_encode_md5    (cherokee_buffer_t  *buf);
+ret_t cherokee_buffer_encoder_hex   (cherokee_buffer_t  *buf);
 
-ret_t  cherokee_buffer_print_debug  (cherokee_buffer_t  *buf, int length);
+ret_t cherokee_buffer_add_version   (cherokee_buffer_t  *buf, int port, cherokee_version_t ver);
+ret_t cherokee_buffer_print_debug   (cherokee_buffer_t  *buf, int length);
 
 #endif /* __CHEROKEE_BUFFER_H__ */
