@@ -26,7 +26,6 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
-#include <dlfcn.h>
 
 #include "common.h"
 #include "log.h"
@@ -36,29 +35,6 @@
 char gbl_buffer[gbl_buffer_size];
 
 
-/* char *  */
-/* stristr (char * s1, char * s2) */
-/* { */
-/* 	   char * ptr = s1; */
-
-/* 	   if (!s1 || !s2 || !*s2) return s1; */
-
-/* 	   while (*ptr) { */
-/* 			 if (TOUPPER(*ptr) == TOUPPER(*s2)) { */
-/* 				    char * cur1 = ptr + 1; */
-/* 				    char * cur2 = s2 + 1; */
-/* 				    while (*cur1 && *cur2 && TOUPPER(*cur1) == TOUPPER(*cur2)) { */
-/* 						  cur1++; */
-/* 						  cur2++; */
-/* 				    } */
-/* 				    if (!*cur2)	return ptr; */
-/* 			 } */
-/* 			 ptr++; */
-/* 	   } */
-/* 	   return NULL; */
-/* } */
-
-
 int
 hexit (char c)
 {
