@@ -38,6 +38,23 @@
 #include <pthread.h>
 #endif
 
+#ifdef HAVE_DLFCN_H
+# include <dlfcn.h>
+#endif
+
+#if defined (HAVE_ENDIAN_H)
+# include <endian.h>
+#elif defined (HAVE_MACHINE_ENDIAN_H)
+# include <machine/endian.h>
+#elif defined (HAVE_SYS_ENDIAN_H)
+# include <sys/endian.h>
+#endif
+
+#ifdef _WIN32
+# include "libc_interface.h"
+#endif
+
+
 #define DEFAULT_RECV_SIZE             1024
 #define DEFAULT_READ_SIZE             8192
 #define MAX_HEADER_LEN                4096
@@ -110,27 +127,50 @@ typedef enum {
 	char obj[size];
 
 
-#ifdef HAVE_PTHREADS
-#define CHEROKEE_MUTEX_LOCK(m)   pthread_mutex_lock(m)
-#define CHEROKEE_MUTEX_UNLOCK(m) pthread_mutex_unlock(m)
+#ifdef HAVE_PTHREAD
+#define CHEROKEE_MUTEX_LOCK(m)    pthread_mutex_lock(m)
+#define CHEROKEE_MUTEX_UNLOCK(m)  pthread_mutex_unlock(m)
+#define CHEROKEE_MUTEX_INIT(m,n)  pthread_mutex_init(m,n)
+#define CHEROKEE_MUTEX_DESTROY(m) pthread_mutex_destroy(m)
 #else
 #define CHEROKEE_MUTEX_LOCK(m)
 #define CHEROKEE_MUTEX_UNLOCK(m)
+#define CHEROKEE_MUTEX_INIT(m,n)  
+#define CHEROKEE_MUTEX_DESTROY(m) 
 #endif
 
 void PRINT_ERROR (const char *format, ...);
 
 
 
+/* cross compilers should define BYTE_ORDER in CPPFLAGS 
+ */
+#ifndef BYTE_ORDER
+
+/* Definitions for byte order, according to byte significance from low
+ * address to high.
+ */
+#define LITTLE_ENDIAN  1234    /* LSB first: i386, vax */
+#define    BIG_ENDIAN  4321    /* MSB first: 68000, ibm, net */
+#define    PDP_ENDIAN  3412    /* LSB first in word, MSW first in long */
+
+/* assume autoconf's AC_C_BIGENDIAN has been ran. If it hasn't, we 
+ * assume (maybe falsely) the order is LITTLE ENDIAN
+ */
+#ifdef WORDS_BIGENDIAN
+# define BYTE_ORDER  BIG_ENDIAN
+#else
+# define BYTE_ORDER  LITTLE_ENDIAN
+#endif
+
+#endif /* BYTE_ORDER */
+
+
+
 /* Misc functions
  */
-
 int   hexit        (char c);
 char *apr_strfsize (size_t size, char *buf);
 
 
-/*                            0        1         2         3         4         5         6         7
-						1234567890123456789012345678901234567890123456789012345678901234567890 */
-
-
 #endif /* __CHEROKEE_COMMON_H__ */
