@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -30,8 +30,6 @@
 #include <pwd.h>
 #include <time.h>
 
-/* inet_ntoa()
- */
 #include <sys/types.h>
 #include <sys/socket.h>
 
@@ -66,19 +64,22 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 	n->arguments   = NULL;
 	n->logger_ref  = NULL;
 	n->keep_alive  = 0;
-	n->timeout     = time(NULL) + 15;   // CHECK ME !!
 	n->range_start = 0;
 	n->range_end   = 0;
 	n->vserver     = NULL;
 	n->log_at_end  = 0;
+	n->realm_ref   = NULL;
 
 	cherokee_buffer_new (&n->local_directory);
 	cherokee_buffer_new (&n->web_directory);
+	cherokee_buffer_new (&n->userdir);
 	cherokee_buffer_new (&n->request);
 	cherokee_buffer_new (&n->redirect);
 	cherokee_buffer_new (&n->host);
 	cherokee_buffer_new (&n->query_string);
-
+	cherokee_buffer_new (&n->user);
+	cherokee_buffer_new (&n->passwd);
+	
 	cherokee_socket_new (&n->socket);
 
 	*cnt = n;
@@ -89,74 +90,81 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 ret_t
 cherokee_connection_free (cherokee_connection_t  *cnt)
 {
-	   cherokee_socket_free (cnt->socket);
-	   cherokee_buffer_free (cnt->buffer);
+	cherokee_socket_free (cnt->socket);
+	cherokee_buffer_free (cnt->buffer);
+	
+	if (cnt->handler != NULL) {
+		cherokee_handler_free (cnt->handler);
+		cnt->handler = NULL;
+	}
 
-	   if (cnt->handler != NULL) {
-		   cherokee_handler_free (cnt->handler);
-		   cnt->handler = NULL;
-	   }
-
-	   if (cnt->encoder != NULL) {
-		   cherokee_encoder_free (cnt->encoder);
-		   cnt->encoder = NULL;
-	   }
-
-	   cherokee_buffer_free (cnt->local_directory);
-	   cherokee_buffer_free (cnt->web_directory);
-	   cherokee_buffer_free (cnt->request);
-	   cherokee_buffer_free (cnt->redirect);
-	   cherokee_buffer_free (cnt->host);
-
-	   if (cnt->arguments) {
-		   cherokee_table_free2 (cnt->arguments, free);
-		   cnt->arguments = NULL;
-	   }
-
-	   free (cnt);
-	   return ret_ok;
+	if (cnt->encoder != NULL) {
+		cherokee_encoder_free (cnt->encoder);
+		cnt->encoder = NULL;
+	}
+	
+	cherokee_buffer_free (cnt->local_directory);
+	cherokee_buffer_free (cnt->web_directory);
+	cherokee_buffer_free (cnt->userdir);
+	cherokee_buffer_free (cnt->request);
+	cherokee_buffer_free (cnt->redirect);
+	cherokee_buffer_free (cnt->host);
+	cherokee_buffer_free (cnt->user);
+	cherokee_buffer_free (cnt->passwd);
+	
+	if (cnt->arguments) {
+		cherokee_table_free2 (cnt->arguments, free);
+		cnt->arguments = NULL;
+	}
+	
+	free (cnt);
+	return ret_ok;
 }
 
 
 ret_t
 cherokee_connection_clean (cherokee_connection_t *cnt)
 {	   
-	   cnt->phase        = phase_reading_header;
-	   cnt->version      = http_version_unknown;
-	   cnt->method       = http_unknown;
-	   cnt->error_code   = http_ok;
-	   cnt->timeout      = -1;
-	   cnt->range_start  = 0;
-	   cnt->range_end    = 0;
-	   cnt->logger_ref   = NULL;
-	   cnt->vserver      = NULL;
-	   cnt->tcp_cork     = 0;
-	   cnt->log_at_end   = 0;
-
-	   if (cnt->handler != NULL) {
-		   cherokee_handler_free (cnt->handler);
-		   cnt->handler = NULL;
-	   }
-
-	   if (cnt->encoder != NULL) {
-		   cherokee_encoder_free (cnt->encoder);
-		   cnt->encoder = NULL;
-	   }
-
-	   cherokee_buffer_clean (cnt->buffer);
-	   cherokee_buffer_clean (cnt->local_directory);
-	   cherokee_buffer_clean (cnt->web_directory);
-	   cherokee_buffer_clean (cnt->request);
-	   cherokee_buffer_clean (cnt->redirect);
-	   cherokee_buffer_clean (cnt->host);
-	   cherokee_buffer_clean (cnt->query_string);
-
-	   if (cnt->arguments) {
-		   cherokee_table_free2 (cnt->arguments, free);
-		   cnt->arguments = NULL;
-	   }
-
-	   return ret_ok;
+	cnt->phase        = phase_reading_header;
+	cnt->version      = http_version_unknown;
+	cnt->method       = http_unknown;
+	cnt->error_code   = http_ok;
+	cnt->timeout      = -1;
+	cnt->range_start  = 0;
+	cnt->range_end    = 0;
+	cnt->logger_ref   = NULL;
+	cnt->vserver      = NULL;
+	cnt->tcp_cork     = 0;
+	cnt->log_at_end   = 0;
+	cnt->realm_ref    = NULL;
+	
+	if (cnt->handler != NULL) {
+		cherokee_handler_free (cnt->handler);
+		cnt->handler = NULL;
+	}
+	
+	if (cnt->encoder != NULL) {
+		cherokee_encoder_free (cnt->encoder);
+		cnt->encoder = NULL;
+	}
+	
+	cherokee_buffer_clean (cnt->buffer);
+	cherokee_buffer_clean (cnt->local_directory);
+	cherokee_buffer_clean (cnt->web_directory);
+	cherokee_buffer_clean (cnt->userdir);
+	cherokee_buffer_clean (cnt->request);
+	cherokee_buffer_clean (cnt->redirect);
+	cherokee_buffer_clean (cnt->host);
+	cherokee_buffer_clean (cnt->query_string);
+	cherokee_buffer_clean (cnt->user);
+	cherokee_buffer_clean (cnt->passwd);
+	
+	if (cnt->arguments) {
+		cherokee_table_free2 (cnt->arguments, free);
+		cnt->arguments = NULL;
+	}
+	
+	return ret_ok;
 }
 
 
@@ -194,6 +202,10 @@ add_error_code_string_to_buffer (cherokee_connection_t *cnt)
 		cherokee_buffer_add (cnt->buffer, http_internal_error_string CRLF, 25); break;
 	case http_moved_permanently: 
 		cherokee_buffer_add (cnt->buffer, http_moved_permanently_string CRLF, 21); break;
+	case http_unauthorized:
+		cherokee_buffer_add (cnt->buffer, http_unauthorized_string CRLF, 26); break;
+	case http_not_modified:
+		cherokee_buffer_add (cnt->buffer, http_not_modified_string CRLF, 16); break;		
 	default:
 		SHOULDNT_HAPPEN;
 	}
@@ -252,6 +264,22 @@ cherokee_connection_send_header (cherokee_connection_t *cnt)
 		}
 	}
 
+	/* Date
+	 */
+	cherokee_buffer_add (cnt->buffer, "Date: ", 6);
+	cherokee_buffer_add_buffer (cnt->buffer, CONN_SRV(cnt)->bogo_now_string);
+	cherokee_buffer_add (cnt->buffer, CRLF, 2);
+
+	/* Authentication
+	 */
+	if ((cnt->realm_ref != NULL)  && 
+	    (cnt->error_code == http_unauthorized)) 
+	{
+		cherokee_buffer_add (cnt->buffer, "WWW-Authenticate: Basic realm=\"", 31);
+		cherokee_buffer_add_buffer (cnt->buffer, cnt->realm_ref);	
+		cherokee_buffer_add (cnt->buffer, "\"" CRLF, 3);	
+	}
+
 	/* Redirected connections
 	 */
 	if (cnt->redirect->len >= 1) {
@@ -259,7 +287,7 @@ cherokee_connection_send_header (cherokee_connection_t *cnt)
 		cherokee_buffer_add (cnt->buffer, cnt->redirect->buf, cnt->redirect->len);
 		cherokee_buffer_add (cnt->buffer, CRLF, 2);
 	}
-	
+
 	/* Encoder headers
 	 */
 	if (cnt->encoder) {
@@ -276,15 +304,15 @@ cherokee_connection_send_header (cherokee_connection_t *cnt)
 	
 	/* Maybe the handler add some headers
 	 */
-	if (cnt->handler) {
+	if ((cnt->handler) && http_type_200(cnt->error_code)) {
 		cherokee_handler_add_headers (cnt->handler, cnt->buffer);
 	}
 	
 	/* Add the "Connection:" header
 	 */
 	if (cnt->handler && cnt->keep_alive) {
-		cherokee_buffer_add (cnt->buffer, "Connection: Keep-Alive"CRLF
-				                  "Keep-Alive: timeout=15"CRLF, 48);
+		cherokee_buffer_add (cnt->buffer, "Connection: Keep-Alive"CRLF, 24);
+		cherokee_buffer_add_buffer (cnt->buffer, CONN_SRV(cnt)->timeout_header);
 	} else {
 		cherokee_buffer_add (cnt->buffer, "Connection: close"CRLF, 19);
 	}
@@ -611,10 +639,11 @@ get_uri (cherokee_connection_t *cnt)
 	
 	/* Look for the QueryString
 	 */
-	cnt->qs_begin_ref = index (cnt->buffer->buf + begin, '?');
-	if (cnt->qs_begin_ref != NULL) {
-		end = cnt->qs_begin_ref - cnt->buffer->buf;
-		cnt->qs_begin_ref++;
+	ptr = index (cnt->buffer->buf + begin, '?');
+
+	if ((ptr) && (ptr < end)) {
+		cnt->qs_begin_ref = ++ptr;
+		end = cnt->qs_begin_ref;
 	}
 
 	/* Dup the request
@@ -643,6 +672,45 @@ get_uri (cherokee_connection_t *cnt)
 }
 
 
+static inline ret_t
+get_authorization (cherokee_connection_t    *cnt,
+		   char                     *ptr)
+{
+	cherokee_buffer_t *auth = NULL;
+
+	if (strncasecmp(ptr, "Basic ", 6) == 0) {
+		char *end;
+
+		ptr += 6;
+		end = strchr (ptr, '\r');
+		if (end == NULL) {
+			goto error;
+		}
+
+		cherokee_buffer_new (&auth);
+		cherokee_buffer_add (auth, ptr, end-ptr+1);
+		cherokee_buffer_decode_base64 (auth);
+
+		end = strchr (auth->buf, ':');
+		if (end == NULL) {
+			goto error;
+		}
+		
+		cherokee_buffer_add (cnt->user, auth->buf, end - auth->buf);
+		cherokee_buffer_add (cnt->passwd, end+1, auth->len  - (end - auth->buf));		
+	}
+
+	return ret_ok;
+
+error:
+	if (auth != NULL) {
+		cherokee_buffer_free (auth);
+	}
+
+	return ret_error;
+}
+
+
 int inline
 cherokee_connection_is_userdir (cherokee_connection_t *cnt)
 {
@@ -658,7 +726,7 @@ cherokee_connection_build_local_directory (cherokee_connection_t *cnt, cherokee_
 	if (entry->document_root->len >= 1) {
 		/* Have a special DocumentRoot
 		 */
-		ret = cherokee_buffer_add (cnt->local_directory, entry->document_root->buf, entry->document_root->len);
+		ret = cherokee_buffer_add_buffer (cnt->local_directory, entry->document_root);
 		
 		/* It has to drop the webdir from the request:
 		 *
@@ -686,45 +754,28 @@ ret_t
 cherokee_connection_build_local_directory_userdir (cherokee_connection_t *cnt, char *userdir)
 {
 	struct passwd *pwd;	
-	char          *begin, *end;
+	char          *begin, *end_username;
 
-	begin = &cnt->request->buf[2];
-	end   = &cnt->request->buf[cnt->request->len-1];
-
-	/* It's an special case. The request is something like:
-	 * 	http://www.alobbs.com/~alo/dir
-	 * and Cherokee should redirect it to:
-	 * 	http://www.alobbs.com/~alo/dir/
+	/* Find user name endding:
 	 */
-	if (*end != '/') {
-		cherokee_buffer_ensure_size (cnt->redirect, cnt->request->len+2);
-		cherokee_buffer_add (cnt->redirect, cnt->request->buf, cnt->request->len);
-		cherokee_buffer_add (cnt->redirect, "/", 1);
+	begin = &cnt->request->buf[2];
 
-		cnt->error_code = http_moved_permanently;
-		return ret_error;
+	end_username = index (begin, '/');
+	if (end_username == NULL) {
+		end_username = &cnt->request->buf[cnt->request->len];
 	}
 
-	/* Find user name endding
-	 */
-	end = index (begin, '/');
-	if (end == NULL)
-		return ret_error;
-		
-	/* Copy the username
-	 */
-	memcpy (gbl_buffer, begin, end-begin);
-	gbl_buffer[end-begin] = '\0';
-	
 	/* Get the user home directory
 	 */
-	pwd = (struct passwd *) getpwnam (gbl_buffer);
+	cherokee_buffer_add (cnt->userdir, begin, end_username - begin);
+
+	pwd = (struct passwd *) getpwnam (cnt->userdir->buf);
 	if ((pwd == NULL) || (pwd->pw_dir == NULL)) {
 		cnt->error_code = http_not_found;
 		return ret_error;
 	}
-		
-	/* Build the local_directory
+
+	/* Build the local_directory:
 	 */
 	cherokee_buffer_add (cnt->local_directory, pwd->pw_dir, strlen(pwd->pw_dir));
 	cherokee_buffer_add (cnt->local_directory, "/", 1);
@@ -732,7 +783,7 @@ cherokee_connection_build_local_directory_userdir (cherokee_connection_t *cnt, c
 
 	/* Drop username from the request
 	 */
-	cherokee_buffer_move_to_begin (cnt->request, (end - cnt->request->buf));
+	cherokee_buffer_move_to_begin (cnt->request, (end_username - cnt->request->buf));
 
 	return ret_ok;
 }
@@ -853,10 +904,63 @@ cherokee_connection_get_plugin_entry (cherokee_connection_t *cnt, cherokee_handl
 		return ret_error;
 	}	
 
+	/* Set the referece to the realm
+	 */
+	cnt->realm_ref = (*plugin_entry)->auth_realm;
+
 	return ret_ok;
 }
 
 
+ret_t 
+cherokee_connection_check_authentication (cherokee_connection_t *cnt, cherokee_handler_table_entry_t *plugin_entry)
+{
+	if (plugin_entry->validator_new_func != NULL) {
+		char *ptr;
+		ret_t ret;
+		cherokee_validator_t *validator = NULL;
+
+		/* Look for authentication in the headers
+		 * It's done on demand because the directory maybe don't have protection
+		 */
+		ptr = strstr (cnt->buffer->buf, "Authorization: ");
+		if (ptr == NULL) {
+			goto unauthorized;
+		}
+		
+		ptr += 15;
+		ret = get_authorization (cnt, ptr);
+		if (ret != ret_ok) {
+			goto unauthorized;
+		}
+		
+		/* Create the validator object
+		 */
+		ret = plugin_entry->validator_new_func (&validator, plugin_entry->properties);
+		if (ret != ret_ok) {
+			cnt->error_code = http_internal_error;
+			return ret_error;	
+		}
+
+		/* Check the login/password
+		 */
+		ret = cherokee_validator_check (validator, cnt);
+		cherokee_validator_free (validator);
+
+		if (ret != ret_ok) {
+			goto unauthorized;
+		}
+       	}
+
+	return ret_ok;
+
+unauthorized:
+	cnt->keep_alive = 0;
+	cnt->error_code = http_unauthorized;
+	return ret_error;
+}
+
+
 ret_t 
 cherokee_connection_create_handler (cherokee_connection_t *cnt, cherokee_handler_table_entry_t *plugin_entry)
 {
@@ -887,6 +991,8 @@ cherokee_connection_parse_header (cherokee_connection_t *cnt, cherokee_encoder_t
 {	   
 	char *ptr;
 
+	printf ("%s\n", cnt->buffer->buf);
+
 	/* Look for "Connection: Keep-Alive / Close"
 	 */
 	if ((ptr = strstr (cnt->buffer->buf, "Connection: "))) {
@@ -930,6 +1036,19 @@ cherokee_connection_parse_header (cherokee_connection_t *cnt, cherokee_encoder_t
 }
 
 
+ret_t 
+cherokee_connection_parse_get_header (cherokee_connection_t *cnt, const char *header, int header_len, char **ret_ref)
+{
+	char *ptr;
+
+	if ((ptr = strstr (cnt->buffer->buf, header))) {
+		*ret_ref = ptr + header_len;
+		return ret_ok;
+	}
+
+	return ret_not_found;
+}
+
 
 /* cherokee_connection_parse_args:
  *  Process the incoming parameters.
@@ -940,18 +1059,27 @@ cherokee_connection_parse_header (cherokee_connection_t *cnt, cherokee_encoder_t
 ret_t 
 cherokee_connection_parse_args (cherokee_connection_t *cnt)
 {
-	int  len;
-	char tmp_end;
+	int   len;
+	ret_t ret;
+	char  tmp_end;
 	char *string, *token;
 
 	if (cnt->qs_begin_ref == NULL) {
 		return ret_ok;
 	}
-
+	
+	/* Argument table is created here, so if it exists is because
+	 * this functions has been called twice.
+	 */
 	if (cnt->arguments != NULL) {
 		return ret_error;
 	}
 
+	/* Create the argument table
+	 */
+	ret = cherokee_table_new (&cnt->arguments);
+	if (ret < ret_ok) return ret;
+
 	/* Look for the end
 	 */
 	len = strcspn (cnt->qs_begin_ref, "\n\r ");
@@ -973,7 +1101,7 @@ cherokee_connection_parse_args (cherokee_connection_t *cnt)
 			val = equ+1;
 
 			// printf ("key='%s' val='%s'\n", key, val);
-			cherokee_table_add (cnt->arguments, key, val);
+			cherokee_table_add (cnt->arguments, key, strdup(val));
 
 			*equ = '=';			
 		} else {
@@ -1036,6 +1164,11 @@ cherokee_connection_send_response_page_hardcoded (cherokee_connection_t *cnt)
 	   case http_moved_permanently:
 		   cherokee_buffer_add (cnt->buffer, "The document has moved", 22);
 		   break;
+	   case http_unauthorized:
+		   cherokee_buffer_add (cnt->buffer, 
+					"This server could not verify that you are authorized to access the document "
+					"requested.  Either you supplied the wrong credentials (e.g., bad password), "
+					"or your browser doesn't understand how to supply the credentials required", 225);
 	   default:
 		   break;
 	   }
