@@ -27,6 +27,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
+#include <time.h>
 
 #include "common.h"
 #include "mime.h"
@@ -54,14 +55,13 @@ cherokee_module_info_t cherokee_file_info = {
 };
 
 ret_t
-cherokee_handler_file_new  (cherokee_handler_t **hdl, void *cnt, cherokee_table_t *properties)
+cherokee_handler_file_new  (cherokee_handler_t **hdl, cherokee_connection_t *cnt, cherokee_table_t *properties)
 {
 	CHEROKEE_NEW_STRUCT (n, handler_file);
 	
 	/* Init the base class object
 	 */
 	cherokee_handler_init_base(HANDLER(n), cnt);
-	HANDLER(n)->connection  = cnt;
 
 	HANDLER(n)->support = hsupport_length | hsupport_range;
 
@@ -80,6 +80,85 @@ cherokee_handler_file_new  (cherokee_handler_t **hdl, void *cnt, cherokee_table_
 	return ret_ok;
 }
 
+static ret_t
+check_cached (cherokee_handler_file_t *n)
+{
+	ret_t ret;
+	char *header;
+	cherokee_connection_t *conn;
+
+	conn = CONN(HANDLER(n)->connection);
+
+		
+	/* Based in time
+	 */
+	ret = cherokee_connection_parse_get_header (conn, "If-Modified-Since: ", 19, &header);
+	if (ret == ret_ok)  {
+		char  *end;
+		time_t req_time;
+		
+		/* Read the date
+		 */
+		end = index (header, '\r');
+		if (end == NULL) {
+			conn->error_code = http_bad_request;
+			return ret_error;		
+		}
+		
+		*end = '\0';   /* on  */
+		req_time = tdate_parse (header);			
+		if (req_time == -1) {
+			cherokee_logger_write_string (
+				CONN_VSRV(conn)->logger, 
+				"Warning: Unparseable time '%s'",
+				header);
+		}
+		*end = '\r';   /* off */
+		
+		if (req_time == -1) {
+			return ret_ok;
+		}
+		
+		
+		/* The file is cached in the client
+		 */
+		if (n->info.st_mtime <= req_time) {
+			conn->error_code = http_not_modified;
+			return ret_error;	
+		}
+	}
+	
+	if (conn->version == http_version_11) {
+		/* Based in ETag
+		 */
+		ret = cherokee_connection_parse_get_header (conn, "If-None-Match: ", 15, &header);
+		if (ret == ret_ok)  {
+			char  *end;
+			int    equal;
+			int    tmp_len;
+			CHEROKEE_TEMP(tmp,100);
+			
+			end = index (header, '\r');
+			if (end == NULL) {
+				conn->error_code = http_bad_request;
+				return ret_error;		
+			}
+			
+			tmp_len = snprintf (tmp, tmp_size, "%lx=%lx", n->info.st_mtime, n->info.st_size);
+
+			*end = '\0';   /* on  */
+			equal = (strncmp (header, tmp, tmp_len) == 0);
+			*end = '\r';   /* off */
+
+			if (equal) {
+				conn->error_code = http_not_modified;
+				return ret_error;				
+			}
+		}
+	}
+
+	return ret_ok;
+}
 
 ret_t 
 cherokee_handler_file_init (cherokee_handler_file_t *n)
@@ -90,7 +169,7 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 
 	conn = CONN(HANDLER(n)->connection);
 
-	cherokee_buffer_add (conn->local_directory, conn->request->buf, conn->request->len); /* (1) */
+	cherokee_buffer_add (conn->local_directory, conn->request->buf+1, conn->request->len-1); /* (1) */
 	n->fd = open (conn->local_directory->buf, O_RDONLY);
 
 	/* Cannot open the file
@@ -101,7 +180,7 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 		ret = stat (conn->local_directory->buf, &n->info);
 	}
 
-	cherokee_buffer_drop_endding (conn->local_directory, conn->request->len); /* (1') */
+	cherokee_buffer_drop_endding (conn->local_directory, conn->request->len);            /* (1') */
 
 
 	if (n->fd == -1) {
@@ -121,7 +200,13 @@ cherokee_handler_file_init (cherokee_handler_file_t *n)
 		return ret_error;		
 	}
 
-
+	/* Is it cached in the client?
+	 */
+	ret = check_cached(n);
+	if (ret != ret_ok) {
+		return ret;
+	}
+	
 	/* Range
 	 */
 	if (conn->range_start != 0) {
@@ -258,6 +343,8 @@ cherokee_handler_file_add_headers (cherokee_handler_file_t *fhdl,
 				   cherokee_buffer_t       *buffer)
 {
 	int length;
+	int len;
+	CHEROKEE_TEMP(tmp,100);
 
 	/* Add "Content-length:" header
 	 */
@@ -287,6 +374,17 @@ cherokee_handler_file_add_headers (cherokee_handler_file_t *fhdl,
 		cherokee_buffer_add (buffer, gbl_buffer, length);		
 	}
 
+	/* Etag
+	 */
+	if (HANDLER_CONN(fhdl)->version >= http_version_11) { 
+		cherokee_buffer_add_va (buffer, "Etag: %lx=%lx"CRLF, fhdl->info.st_mtime, fhdl->info.st_size);
+	}
+
+	/* Last-Modified:
+	 */
+	len = strftime (tmp, tmp_size, "Last-Modified: %a, %d %b %Y %H:%M:%S GMT"CRLF, gmtime(&fhdl->info.st_mtime));
+	cherokee_buffer_add (buffer, tmp, len);
+
 	return ret_ok;
 }
 
