@@ -34,6 +34,7 @@
 #include "common.h"
 #include "buffer.h"
 #include "handler.h"
+#include "connection.h"
 
 
 typedef struct {
@@ -58,7 +59,7 @@ typedef struct {
 void file_init ();
 
 
-ret_t cherokee_handler_file_new   (cherokee_handler_t **hdl, void *cnt, cherokee_table_t *properties);
+ret_t cherokee_handler_file_new   (cherokee_handler_t **hdl, cherokee_connection_t *cnt, cherokee_table_t *properties);
 
 /* virtual methods implementation
  */
