@@ -27,6 +27,12 @@
 #include "connection.h"
 
 
+cherokee_module_info_t cherokee_gnomevfs_info = {
+	cherokee_handler,                /* type     */
+	cherokee_handler_gnomevfs_new    /* new func */
+};
+
+
 ret_t 
 cherokee_handler_gnomevfs_new (cherokee_handler_t **hdl, void *cnt, cherokee_table_t *properties)
 {
@@ -40,10 +46,10 @@ cherokee_handler_gnomevfs_new (cherokee_handler_t **hdl, void *cnt, cherokee_tab
 
 	HANDLER(n)->support = hsupport_length | hsupport_range;
 
-	HANDLER(n)->init        = (func_init_t) cherokee_handler_gnomevfs_init;
-	HANDLER(n)->free        = (func_free_t) cherokee_handler_gnomevfs_free;
-	HANDLER(n)->step        = (func_step_t) cherokee_handler_gnomevfs_step;
-	HANDLER(n)->add_headers = (func_add_headers_t) cherokee_handler_gnomevfs_add_headers;
+	HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_gnomevfs_init;
+	HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_gnomevfs_free;
+	HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_gnomevfs_step;
+	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_gnomevfs_add_headers;
 
 	*hdl = HANDLER(n);
 	
