@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2002 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -114,10 +114,10 @@ cherokee_handler_php4_new (cherokee_handler_t **hdl, void *cnt, cherokee_table_t
 
 //	HANDLER(n)->support = hsupport_length | hsupport_range;
 
-	HANDLER(n)->init        = (func_init_t) cherokee_handler_php4_init;
-	HANDLER(n)->free        = (func_free_t) cherokee_handler_php4_free;
-	HANDLER(n)->step        = (func_step_t) cherokee_handler_php4_step;
-	HANDLER(n)->add_headers = (func_add_headers_t) cherokee_handler_php4_add_headers;
+	HANDLER(n)->init        = (handler_func_init_t) cherokee_handler_php4_init;
+	HANDLER(n)->free        = (handler_func_free_t) cherokee_handler_php4_free;
+	HANDLER(n)->step        = (handler_func_step_t) cherokee_handler_php4_step;
+	HANDLER(n)->add_headers = (handler_func_add_headers_t) cherokee_handler_php4_add_headers;
 
 	*hdl = HANDLER(n);
 	
