@@ -45,7 +45,9 @@ typedef enum {
 	   http_accepted          = 202,
 	   http_partial_content   = 206,
 	   http_moved_permanently = 301,
+	   http_not_modified      = 304,
 	   http_bad_request       = 400,
+	   http_unauthorized      = 401,
 	   http_access_denied     = 403,
 	   http_not_found         = 404,
 	   http_internal_error    = 500
@@ -57,7 +59,9 @@ typedef enum {
 #define http_accepted_string           "202 Accepted"
 #define http_partial_content_string    "206 Partial Content"
 #define http_moved_permanently_string  "301 Moved Permanently"
+#define http_not_modified_string       "304 Not Modified"
 #define http_bad_request_string        "400 Bad Request"
+#define http_unauthorized_string       "401 Authorization Required"
 #define http_access_denied_string      "403 Forbidden"
 #define http_not_found_string          "404 Not Found"
 #define http_internal_error_string     "500 Internal Server Error"
