@@ -11,7 +11,7 @@
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  * 
  *
- * Copyright (C) 2001-2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,10 +47,6 @@
 #include "server.h"
 #include "module.h"
 
-#ifdef HAVE_PTHREAD
-pthread_mutex_t buffer_lock = PTHREAD_MUTEX_INITIALIZER;
-#endif
-
 
 /* Some constants
  */
@@ -145,16 +141,16 @@ cherokee_logger_ncsa_free (cherokee_logger_ncsa_t *logger)
 		ret = fclose (logger->errorlog_fd);
 		n--;
 	}
-
+	
 	if (logger->accesslog_fd != NULL) {
 		ret |= fclose (logger->errorlog_fd);
 		n--;
 	}
-
+	
 	if (n != 0) {
 		closelog();
 	}
-
+	
 	free (logger);
 	
 	return (ret == 0) ? ret_ok : ret_error;
@@ -164,31 +160,30 @@ cherokee_logger_ncsa_free (cherokee_logger_ncsa_t *logger)
 ret_t 
 cherokee_logger_ncsa_flush (cherokee_logger_ncsa_t *logger)
 {
+	int   tmp;
 	ret_t ret = ret_ok;
 
-	CHEROKEE_MUTEX_LOCK(&buffer_lock);
-
-	if (cherokee_buffer_is_empty(LOGGER_BUFFER(logger))) {
+	if (cherokee_buffer_is_empty (LOGGER_BUFFER(logger))) {
 		return ret_ok;
 	}
 
-	if (logger->accesslog_fd != NULL) {
-		int tmp;
-
-		tmp = fwrite (LOGGER(logger)->buffer->buf, 1, LOGGER_BUFFER(logger)->len, logger->accesslog_fd); 
-                fflush(logger->accesslog_fd);
-
-		ret = (tmp > 0) ? ret_ok : ret_error;
-
-	} else {
-		syslog (LOG_INFO, LOGGER(logger)->buffer->buf);
+	if (logger->accesslog_fd == NULL) {
+		syslog (LOG_INFO, LOGGER_BUFFER(logger)->buf);
+		return cherokee_buffer_make_empty (LOGGER_BUFFER(logger));
 	}
 
-	cherokee_buffer_make_empty (LOGGER(logger)->buffer);
 
-	CHEROKEE_MUTEX_UNLOCK(&buffer_lock);
+	tmp = fwrite (LOGGER_BUFFER(logger)->buf, 1, LOGGER_BUFFER(logger)->len, logger->accesslog_fd); 
+	fflush (logger->accesslog_fd);
+	if (tmp < 0) {
+		return ret_error;
+	}
 
-	return ret;
+	if (tmp == LOGGER_BUFFER(logger)->len) {
+		return cherokee_buffer_make_empty (LOGGER_BUFFER(logger));
+	} 
+
+	return cherokee_buffer_drop_endding (LOGGER_BUFFER(logger), tmp);
 }
 
 
@@ -272,7 +267,7 @@ cherokee_logger_ncsa_write_access (cherokee_logger_ncsa_t *logger, cherokee_conn
 	int  len;
 	struct tm conn_time;
 	char ipaddr[INET6_ADDRSTRLEN];
-	CHEROKEE_TEMP (tmp, 200);
+	CHEROKEE_TEMP (tmp, 400);
     
 	/* thread-safe version of localtime 
 	 */
@@ -306,9 +301,7 @@ cherokee_logger_ncsa_write_access (cherokee_logger_ncsa_t *logger, cherokee_conn
 			cnt->error_code,
 			(long) cnt->range_end - cnt->range_start);
 	
-	CHEROKEE_MUTEX_LOCK(&buffer_lock);
-	ret = cherokee_buffer_add (LOGGER(logger)->buffer, tmp, len);
-	CHEROKEE_MUTEX_UNLOCK(&buffer_lock);
+	ret = cherokee_buffer_add (LOGGER_BUFFER(logger), tmp, len);
 
 	return ret;
 }
