@@ -37,6 +37,7 @@
 #include <time.h>
 #endif
 
+#include "module.h"
 #include "logger_w3c.h"
 #include "connection.h"
 #include "server.h"
@@ -59,6 +60,12 @@ static char *month[]   = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 #define IN_ADDR(c) ((struct in_addr) (c).sin_addr)
 
 
+cherokee_module_info_t cherokee_w3c_info = {
+	cherokee_logger,            /* type     */
+	cherokee_logger_w3c_new     /* new func */
+};
+
+
 ret_t
 cherokee_logger_w3c_new  (cherokee_logger_t **logger, cherokee_table_t *properties)
 {
