@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -45,6 +45,8 @@
 #include "thread.h"
 #endif
 
+static ret_t init_tls (void);
+
 
 ret_t
 cherokee_server_new  (cherokee_server_t **srv)
@@ -58,8 +60,9 @@ cherokee_server_new  (cherokee_server_t **srv)
 	
 	/* Sockets
 	 */
-	n->socket = -1;
-	n->ipv6   =  1;
+	n->socket     = -1;
+	n->socket_tls = -1;
+	n->ipv6       =  1;
 	
 	/* Threads
 	 */
@@ -73,14 +76,14 @@ cherokee_server_new  (cherokee_server_t **srv)
 	/* Server config
 	 */
 	n->port           = 80;
+	n->port_tls       = 443;
 	n->listen_to      = NULL;
 	n->fdwatch_msecs  = 999;
-	n->timeout        = 15;
 	n->hideversion    = 0;
 	n->hideservername = 0;
-	n->mimetypes_file = NULL;
 	n->userdir        = NULL;
 	n->userdir_handler= NULL;
+	n->icons_file     = NULL;
 
 	n->keepalive      = 1;
 	n->keepalive_max  = MAX_KEEP_ALIVE;
@@ -96,6 +99,13 @@ cherokee_server_new  (cherokee_server_t **srv)
 	n->group_orig     = getgid();
 	n->group          = n->group_orig;
 
+	n->timeout        = 15;
+	cherokee_buffer_new (&n->timeout_header);
+	cherokee_buffer_add (n->timeout_header, "Keep-Alive: timeout=15"CRLF, 48);
+
+
+	cherokee_buffer_new (&n->bogo_now_string);
+	cherokee_buffer_ensure_size (n->bogo_now_string, 100);
 
 	/* Virtual servers table
 	 */
@@ -115,7 +125,11 @@ cherokee_server_new  (cherokee_server_t **srv)
 	cherokee_encoder_table_new (&n->encoders);
 	return_if_fail (n->encoders != NULL, ret_nomem);
 
-	
+	/* Icons 
+	 */
+	cherokee_icons_new (&n->icons);
+	return_if_fail (n->icons != NULL, ret_nomem);
+
 	/* Loggers
 	 */
 	n->log_flush_next   = 0;
@@ -124,6 +138,7 @@ cherokee_server_new  (cherokee_server_t **srv)
 	cherokee_logger_table_new (&n->loggers);
 	return_if_fail (n->loggers != NULL, ret_nomem);
 
+	init_tls();
 
 	/* Return the object
 	 */
@@ -133,6 +148,29 @@ cherokee_server_new  (cherokee_server_t **srv)
 }
 
 
+static ret_t
+init_tls (void)
+{
+#ifdef HAVE_TLS
+	int rc;
+
+	rc = gnutls_global_init();
+	if (rc < 0) {
+		PRINT_ERROR ("Global TLS state initialisation failed.\n");
+		return ret_error;
+	}
+
+	rc = gnutls_global_init_extra();
+	if (rc < 0) {
+		PRINT_ERROR ("Global TLS state initialisation `extra' failed.\n");
+		return ret_error;
+	}	
+#endif
+
+	return ret_ok;
+}
+
+
 static inline ret_t
 change_execution_user (cherokee_server_t *srv)
 {
@@ -170,30 +208,33 @@ cherokee_server_set_min_latency (cherokee_server_t *srv, int msecs)
 
 
 static ret_t
-initialize_server_socket4 (cherokee_server_t *srv)
+initialize_server_socket4 (cherokee_server_t *srv, unsigned short port, int *srv_socket_ret)
 {
-	ret_t ret;
-	int   on = 1;
+	int re;
+	int on = 1;
+	int srv_socket;
 	struct sockaddr_in  sockaddr;
 	
-	srv->socket = socket (AF_INET, SOCK_STREAM, 0);
-	if (srv->socket <= 0) {
+	srv_socket = socket (AF_INET, SOCK_STREAM, 0);
+	if (srv_socket <= 0) {
 		PRINT_ERROR ("Error creating IPv4 server socket\n");
 		exit(EXIT_CANT_CREATE_SERVER_SOCKET4);
 	}
 
+	*srv_socket_ret = srv_socket;
+
 	/* Set 'close-on-exec'
 	 */
-	fcntl(srv->socket, F_SETFD, FD_CLOEXEC);
+	fcntl (srv_socket, F_SETFD, FD_CLOEXEC);
 		
 	/* To re-bind without wait to TIME_WAIT
 	 */
-	ret = setsockopt (srv->socket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
-	return_if_fail (ret == 0, ret_error);
+	re = setsockopt (srv_socket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
+	if (re != 0) return ret_error;
 
 	/* Bind the socket
 	 */
-	sockaddr.sin_port   = htons (srv->port);
+	sockaddr.sin_port   = htons (port);
 	sockaddr.sin_family = AF_INET;
 
 	if (srv->listen_to == NULL) {
@@ -209,42 +250,46 @@ initialize_server_socket4 (cherokee_server_t *srv)
 #endif
 	}
 
-	ret = bind (srv->socket, (struct sockaddr *)&sockaddr, sizeof(sockaddr));
-	return ret;
+	re = bind (srv_socket, (struct sockaddr *)&sockaddr, sizeof(sockaddr));
+	return (re == 0) ? ret_ok : ret_error;
 }
 
 
 static ret_t
-initialize_server_socket6 (cherokee_server_t *srv)
+initialize_server_socket6 (cherokee_server_t *srv, unsigned short port, int *srv_socket_ret)
 {
 #ifdef HAVE_IPV6
-	ret_t ret;
-	int   on = 1;
+	int re;
+	int on = 1;
+	int srv_socket;
 	struct sockaddr_in6 sockaddr;
 
 
 	/* Create the socket
 	 */
-	srv->socket = socket (AF_INET6, SOCK_STREAM, 0);
-	if (srv->socket <= 0) {
+	srv_socket = socket (AF_INET6, SOCK_STREAM, 0);
+	if (srv_socket <= 0) {
 		PRINT_ERROR ("Error creating IPv6 server socket.. switching to IPv4\n");
 		srv->ipv6 = 0;
 		return ret_error;
 	}
 
+
+	*srv_socket_ret = srv_socket;
+
 	/* Set 'close-on-exec'
 	 */
-	fcntl(srv->socket, F_SETFD, FD_CLOEXEC);
+	fcntl (srv_socket, F_SETFD, FD_CLOEXEC);
 	
 	/* To re-bind without wait to TIME_WAIT
 	 */
-	ret = setsockopt (srv->socket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
-	return_if_fail (ret == 0, ret_error);
+	re = setsockopt (srv_socket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
+	if (re != 0) return ret_error;
 
 	/* Bind the socket
 	 */
-	sockaddr.sin6_port   = htons (srv->port);
-	sockaddr.sin6_family = AF_INET6;		
+	sockaddr.sin6_port     = htons (port);  /* Transport layer port #   */
+	sockaddr.sin6_family   = AF_INET6;      /* AF_INET6                 */
 
 	if (srv->listen_to == NULL) {
 		sockaddr.sin6_addr = in6addr_any; 
@@ -252,58 +297,59 @@ initialize_server_socket6 (cherokee_server_t *srv)
 		inet_pton (sockaddr.sin6_family, srv->listen_to, &sockaddr.sin6_addr);
 	}
 
-	ret = bind (srv->socket, (struct sockaddr *)&sockaddr, sizeof(sockaddr));
-	return ret;
+	re = bind (srv_socket, (struct sockaddr *)&sockaddr, sizeof(sockaddr));
+	return (re == 0) ? ret_ok : ret_error;
 #endif
 }
 
 
 static ret_t
-initialize_server_socket (cherokee_server_t *srv)
+initialize_server_socket (cherokee_server_t *srv, unsigned short port, int *srv_socket_ptr)
 {
+	ret_t ret;
 	int   flags;
-	ret_t ret = ret_ok;
+	int   srv_socket;
 
 #ifdef HAVE_IPV6
 	if (srv->ipv6) {
-		ret = initialize_server_socket6 (srv);
+		ret = initialize_server_socket6 (srv, port, srv_socket_ptr);
 	} 
 #endif
 	if ((ret != ret_ok) || (srv->ipv6 == 0)) {
-		ret = initialize_server_socket4 (srv);		
+		ret = initialize_server_socket4 (srv, port, srv_socket_ptr);
 	}
 
+	srv_socket = *srv_socket_ptr;
+
 	if (ret != 0) {
 		uid_t uid = getuid();
 		gid_t gid = getgid();
 
-		PRINT_ERROR ("Can't bind() socket (port=%d, UID=%d, GID=%d)\n", srv->port, uid, gid);
+		PRINT_ERROR ("Can't bind() socket (port=%d, UID=%d, GID=%d)\n", port, uid, gid);
 		return ret_error;
 	}
 
-	printf ("Listening on port %d\n", srv->port);
+	printf ("Listening on port %d\n", port);
 	   
 	/* Set no-delay mode
 	 */
-	flags = fcntl (srv->socket, F_GETFL, 0);
+	flags = fcntl (srv_socket, F_GETFL, 0);
 	return_if_fail (flags != -1, ret_error);
 	
-	ret = fcntl (srv->socket, F_SETFL, flags | O_NDELAY);
+	ret = fcntl (srv_socket, F_SETFL, flags | O_NDELAY);
 	return_if_fail (ret >= 0, ret_error);
 	
-	
 	/* Listen
 	 */
-	ret = listen (srv->socket, 1024);
+	ret = listen (srv_socket, 1024);
 	if (ret < 0) {
-		close (srv->socket);
+		close (srv_socket);
 		return_if_fail (0, ret_error);
 	}
 
-
 	/* Add the socket to fdpoll
 	 */
-	cherokee_fdpoll_add (srv->main_thread->fdpoll, srv->socket, 0);
+	cherokee_fdpoll_add (srv->main_thread->fdpoll, srv_socket, 0);
 
 	return ret_ok;
 }
@@ -335,6 +381,15 @@ initialize_server_threads (cherokee_server_t *srv)
 }
 
 
+#ifdef HAVE_TLS
+static void
+for_each_vserver_init_tls_func (const char *key, void *value)
+{
+	cherokee_virtual_server_init_tls (VSERVER(value));
+}
+#endif
+
+
 ret_t
 cherokee_server_init (cherokee_server_t *srv) 
 {   
@@ -344,10 +399,24 @@ cherokee_server_init (cherokee_server_t *srv)
 	 * because a SIGHUP, it shouldn't init the server socket.
 	 */
 	if (srv->socket == -1) {
-		ret = initialize_server_socket (srv);
+		ret = initialize_server_socket (srv, srv->port, &srv->socket);
 		if (ret != ret_ok) return ret;
 	}
 
+#ifdef HAVE_TLS
+	/* Initialize the server TLS socket
+	 */
+	if (srv->socket_tls == -1) {
+		ret = initialize_server_socket (srv, srv->port_tls, &srv->socket_tls);
+		if (ret != ret_ok) return ret;
+	}
+
+	/* Initialize TLS in all the virtual servers
+	 */
+	cherokee_virtual_server_init_tls (srv->vserver_default);
+	cherokee_table_foreach (srv->vservers, for_each_vserver_init_tls_func);
+#endif
+
         /* Threads / CPUs
 	 */
 	if (srv->ncpus == -1) {
@@ -415,11 +484,6 @@ cherokee_server_clean (cherokee_server_t *srv)
 		srv->listen_to = NULL;
 	}
 
-	if (srv->mimetypes_file != NULL) {
-		free (srv->mimetypes_file);
-		srv->mimetypes_file = NULL;
-	}
-
 	if (srv->userdir != NULL) {
 		free (srv->userdir);
 		srv->userdir = NULL;
@@ -434,10 +498,16 @@ cherokee_server_clean (cherokee_server_t *srv)
 		free (srv->chroot);
 		srv->chroot = NULL;
 	}
+
+	if (srv->icons_file != NULL) {
+		free (srv->icons_file);
+		srv->icons_file = NULL;
+	}
 	
 	/* Clean the loader
 	 */
-	// TODO
+	cherokee_module_loader_free (srv->loader);
+	srv->loader = NULL;
 
 	/* Clean the encoders table
 	 */
@@ -456,6 +526,14 @@ cherokee_server_clean (cherokee_server_t *srv)
 	cherokee_table_foreach (srv->vservers, for_each_func_free_vserver);
 	cherokee_table_clean (srv->vservers);
 
+	/* Icons 
+	 */
+	cherokee_icons_clean (srv->icons);
+
+	/* Time-out pre-builded header
+	 */
+	cherokee_buffer_clean (srv->timeout_header);
+
 	/* Clean the index list
 	 */
 	cherokee_list_free (&srv->index_list, free);
@@ -475,6 +553,10 @@ cherokee_server_free (cherokee_server_t *srv)
 {
 	close (srv->socket);
 
+	if (srv->socket_tls != -1) {
+		close (srv->socket_tls);		
+	}
+
 	cherokee_encoder_table_free (srv->encoders);
 	cherokee_logger_table_free (srv->loggers);
 
@@ -490,16 +572,16 @@ cherokee_server_free (cherokee_server_t *srv)
 
 	cherokee_module_loader_free (srv->loader);
 
+	cherokee_icons_free (srv->icons);
+	cherokee_buffer_free (srv->bogo_now_string);
+
+	cherokee_buffer_free (srv->timeout_header);
+
 	if (srv->listen_to != NULL) {
 		free (srv->listen_to);
 		srv->listen_to = NULL;
 	}
 
-	if (srv->mimetypes_file != NULL) {
-		free (srv->mimetypes_file);
-		srv->mimetypes_file = NULL;
-	}
-
 	if (srv->userdir != NULL) {
 		free (srv->userdir);
 		srv->userdir = NULL;
@@ -520,6 +602,11 @@ cherokee_server_free (cherokee_server_t *srv)
 		srv->config_filename = NULL;
 	}
 
+	if (srv->icons_file != NULL) {
+		free (srv->icons_file);
+		srv->icons_file = NULL;
+	}
+
 	free (srv);
 	
 	return ret_ok;
@@ -527,7 +614,7 @@ cherokee_server_free (cherokee_server_t *srv)
 
 
 static inline int
-look_for_new_connection (cherokee_server_t *srv)
+look_for_new_connection (cherokee_server_t *srv, int srv_socket, int is_tls)
 {
 	ret_t   ret;
 	list_t *thread;
@@ -538,7 +625,7 @@ look_for_new_connection (cherokee_server_t *srv)
 
 	/* Return if there're no new connections
 	 */
-	if (cherokee_fdpoll_check (srv->main_thread->fdpoll, srv->socket, 0) == 0) {
+	if (cherokee_fdpoll_check (srv->main_thread->fdpoll, srv_socket, 0) == 0) {
 		return 0;
 	}
 
@@ -562,17 +649,28 @@ look_for_new_connection (cherokee_server_t *srv)
 
 	/* Accept the connection in the new socket
 	 */
-	ret = cherokee_socket_accept (CONN_SOCK(new_conn), srv->socket);
-	if (ret < ret_ok) {
-		cherokee_thread_set_reusable_connection (lazy_thread, new_conn);
-		return ret;
+	ret = cherokee_socket_accept (CONN_SOCK(new_conn), srv_socket);
+	if (ret < ret_ok) goto error;
+
+	/* May active the TLS support
+	 */
+	if (is_tls) {
+		new_conn->phase = phase_tls_handshake;
 	}
 
 	/* Add the configured connection to the thread
 	 */
 	cherokee_thread_add_connection (lazy_thread, new_conn);
 
+	/* Set the connection time-out
+	 */
+	new_conn->timeout =  srv->bogo_now + srv->timeout;
+
 	return 1;
+
+error:
+	cherokee_thread_set_reusable_connection (lazy_thread, new_conn);
+	return 0;
 }
 
 
@@ -601,8 +699,6 @@ cherokee_server_reinit (cherokee_server_t *srv)
 {
 	ret_t  ret;
 	
-	printf ("cherokee_server_reinit\n");
-
 	if (srv->chrooted) {
 		fprintf (stderr, 
 			 "WARNING: Chrooted cherokee cannot be reloaded. "
@@ -626,30 +722,60 @@ cherokee_server_reinit (cherokee_server_t *srv)
 		exit(EXIT_SERVER_INIT);	
 	}
 
-	printf ("cherokee_server_reinit2\n");
-
 	return ret_ok;
 }
 
 
+static void
+update_bogo_now (cherokee_server_t *srv)
+{
+	static unsigned char to_wait = 1;
+
+ 	srv->bogo_now = time (NULL);
+	
+	if (--to_wait == 0) {
+		srv->bogo_now_string->len = strftime (srv->bogo_now_string->buf, srv->bogo_now_string->size,
+						      "%a, %d %b %Y %H:%M:%S GMT", gmtime(&srv->bogo_now));
+		to_wait = 1000 / srv->fdwatch_msecs;
+	}
+}
+
+
+static void
+process_new_connections (cherokee_server_t *srv, int srv_socket, int is_tls)
+{
+	ret_t ret;
+ 	int   i;
+
+	i = MAX_NEW_CONNECTIONS_PER_STEP; 
+
+	/* Watch the sockets, look for new connections
+	 */
+	do {
+		cherokee_fdpoll_reset (srv->main_thread->fdpoll, srv_socket);
+		cherokee_fdpoll_watch (srv->main_thread->fdpoll, srv->fdwatch_msecs);
+
+		ret = look_for_new_connection (srv, srv_socket, is_tls);
+
+	} while (ret && (i-- > 0));
+}
+
 void
 cherokee_server_step (cherokee_server_t *srv)
 {
 	ret_t ret;
- 	int   i   = MAX_NEW_CONNECTIONS_PER_STEP; 
 
-	/* Watch the sockets, look for new connections
+	/* Process the new connections
 	 */
-	do {
-		cherokee_fdpoll_reset (srv->main_thread->fdpoll, srv->socket);
-		cherokee_fdpoll_watch (srv->main_thread->fdpoll, srv->fdwatch_msecs);
-		ret = look_for_new_connection(srv);
-	} while (ret && (i-- > 0));
+	process_new_connections (srv, srv->socket, 0);
 
+#ifdef HAVE_TLS	
+	process_new_connections (srv, srv->socket_tls, 1);
+#endif	
 
 	/* Get the time
 	 */
- 	srv->bogo_now = time (NULL); 
+	update_bogo_now (srv);
 	ret = cherokee_thread_step (srv->main_thread);
 
 	/* Logger flush 
@@ -672,6 +798,7 @@ cherokee_server_step (cherokee_server_t *srv)
 ret_t 
 cherokee_server_read_config_file (cherokee_server_t *srv, char *filename)
 {
+	ret_t ret;
 	int   error;
 	void *bufstate;
 	extern FILE *yyin;
@@ -706,22 +833,35 @@ cherokee_server_read_config_file (cherokee_server_t *srv, char *filename)
 	 */
 	yyrestart(yyin);
 
-//	error = yyparse ((void *)srv);
-	
 	bufstate = (void *) yy_create_buffer (yyin, 65535);
-	yy_switch_to_buffer(bufstate);
+	yy_switch_to_buffer (bufstate);
 	error = yyparse ((void *)srv);
 	yy_delete_buffer (bufstate);
 
 	fclose (yyin);
 
-	return (error)? ret_error : ret_ok;
+	ret = (error == 0) ? ret_ok : ret_error;
+	if (ret < ret_ok) {
+		return ret;
+	}
+
+	/* Maybe read /etc/cherokee/icons.conf
+	 */
+	if (srv->icons_file != NULL) {
+		ret = cherokee_icons_read_config_file (srv->icons, srv->icons_file);
+		if (ret < ret_ok) {
+			PRINT_ERROR ("Cannot read the icons file\n");
+		}
+	}
+
+	return ret;
 }
 
 
 ret_t 
 cherokee_server_read_config_string (cherokee_server_t *srv, char *config_string)
 {
+	ret_t ret;
 	int   error;
 	void *bufstate;
 
@@ -737,7 +877,21 @@ cherokee_server_read_config_string (cherokee_server_t *srv, char *config_string)
 
 	yy_delete_buffer (bufstate);
 
-	return (error)? ret_error : ret_ok;
+	ret = (error == 0) ? ret_ok : ret_error;
+	if (ret < ret_ok) {
+		return ret;
+	}
+	
+        /* Maybe read /etc/cherokee/icons.conf
+	 */
+	if (srv->icons_file != NULL) {
+		ret = cherokee_icons_read_config_file (srv->icons, srv->icons_file);
+		if (ret < ret_ok) {
+			PRINT_ERROR ("Cannot read the icons file\n");
+		}
+	}
+
+	return ret;
 }
 
 
@@ -745,7 +899,6 @@ ret_t
 cherokee_server_handle_HUP (cherokee_server_t *srv)
 {
 	srv->wanna_exit = 1;
-	printf ("cherokee_server_handle_HUP\n");
 	return ret_ok;
 }
 
