@@ -44,10 +44,14 @@
 #include "logger_table.h"
 #include "thread.h"
 #include "module_loader.h"
+#include "icons.h"
 
 
 typedef struct {
-	time_t bogo_now;
+	/* Current time
+	 */
+	time_t             bogo_now;
+	cherokee_buffer_t *bogo_now_string;
 
 	int    wanna_exit;
 	char  *config_filename;
@@ -68,6 +72,10 @@ typedef struct {
 	cherokee_encoder_table_t *encoders;
 	cherokee_logger_table_t  *loggers;
 
+	/* Icons
+	 */
+	cherokee_icons_t *icons;
+
 	/* Default index files
 	 */
 	list_t index_list;
@@ -80,6 +88,11 @@ typedef struct {
 	/* Configuration
 	 */
 	int   socket;
+	int   socket_tls;
+
+	unsigned short port;
+	unsigned short port_tls;
+
 	int   ipv6;
 	char *listen_to;	
 	int   fdwatch_msecs;
@@ -87,8 +100,9 @@ typedef struct {
 	int   ncpus;
 	int   thread_num;
 
-	int   port;
-	int   timeout;
+	int                timeout;
+	cherokee_buffer_t *timeout_header;
+
 	int   hideversion;
 	int   hideservername;
 
@@ -104,6 +118,7 @@ typedef struct {
 	int    chrooted;
 
 	char *mimetypes_file;
+	char *icons_file;
 
 	char                           *userdir;              /* Eg: public_html */
 	cherokee_handler_table_entry_t *userdir_handler;     
