@@ -36,9 +36,23 @@
 #endif
 
 #include <sys/types.h>	
-#include <sys/socket.h>
-#include <arpa/inet.h>
-#include <netinet/in.h>
+
+#ifdef HAVE_SYS_SOCKET_H
+# include <sys/socket.h>
+#endif
+
+#ifdef HAVE_ARPA_INET_H
+# include <arpa/inet.h>
+#endif
+
+#ifdef HAVE_NETINET_IN_H
+# include <netinet/in.h>
+#endif
+
+#ifdef HAVE_TLS
+# include <gnutls/extra.h>
+# include <gnutls/gnutls.h>
+#endif
 
 #include "list.h"
 #include "http.h"
@@ -62,6 +76,11 @@ typedef struct {
 	struct sockaddr           addr_in;
 	int                       addr_len;
 	cherokee_socket_status_t  status;
+	int                       is_tls;
+
+#ifdef HAVE_TLS	
+	gnutls_session session;
+#endif
 } cherokee_socket_t;
 
 #define SOCKET(s)        ((cherokee_socket_t *)(s))
@@ -70,19 +89,16 @@ typedef struct {
 #define SOCKET_ADDR(s)   (SOCKET(s)->addr_in)
 #define SOCKET_STATUS(s) (SOCKET(s)->status)
 
-/*
-#define CHEROKEE_SOCKET_FD(s)     (SOCKET(s)->socket)
-#define CHEROKEE_SOCKET_ADDR(s)   (SOCKET(s)->addr_in)
-#define CHEROKEE_SOCKET_AF(s)     (SOCKET(s)->addr_in.sa_family)
-#define CHEROKEE_SOCKET_STATUS(s) (SOCKET(s)->status)
-*/
 
+ret_t       cherokee_socket_new        (cherokee_socket_t **socket);
+ret_t       cherokee_socket_free       (cherokee_socket_t  *socket);
 
-ret_t       cherokee_socket_new    (cherokee_socket_t **socket);
-ret_t       cherokee_socket_free   (cherokee_socket_t  *socket);
-
+ret_t       cherokee_socket_init_tls   (cherokee_socket_t  *socket, cherokee_virtual_server_t *vserver);
 ret_t       cherokee_socket_close      (cherokee_socket_t  *socket);
 ret_t       cherokee_socket_accept     (cherokee_socket_t  *socket, int server_socket);
+ret_t       cherokee_socket_write      (cherokee_socket_t  *socket, cherokee_buffer_t *buf);
+ret_t       cherokee_socket_read       (cherokee_socket_t  *socket, cherokee_buffer_t *buf);
+
 const char *cherokee_socket_ntop       (cherokee_socket_t  *socket, char*, size_t);
 ret_t       cherokee_socket_set_status (cherokee_socket_t  *socket, cherokee_socket_status_t);
 
