@@ -23,18 +23,43 @@
  * USA
  */
 
+#include <signal.h>
+#include <errno.h>
+
 #include "thread.h"
 #include "server.h"
 #include "socket.h"
 #include "connection.h"
 
 
+static ret_t
+set_sigpipe (void)
+{
+       struct sigaction new_act;
+       struct sigaction old_act;
+
+       memset (&new_act, 0, sizeof( new_act ));
+       memset (&old_act, 0, sizeof( old_act ));
+
+       new_act.sa_handler = SIG_IGN;
+
+       if (sigaction (SIGPIPE, &new_act, &old_act)) {
+               PRINT_ERROR ("sigaction (SIGPIPE, ...): %s (%d)\n", strerror( errno ), errno);
+	       return ret_error;
+       }
+
+       return ret_ok;
+}
+
+
 #ifdef HAVE_PTHREAD
 static void *
 thread_routine (void *data)
 {
         cherokee_thread_t *thread = THREAD(data);
-        cherokee_server_t *srv = THREAD(data)->server;
+        cherokee_server_t *srv    = THREAD_SRV(data);
+
+	set_sigpipe();	
 
         while (thread->exit == 0) {
                 cherokee_fdpoll_watch (thread->fdpoll, srv->fdwatch_msecs);
@@ -62,7 +87,9 @@ cherokee_thread_new  (cherokee_thread_t **thd, void *server, int real_thread)
 	n->active_list_num = 0;
 	n->server = server;
 	n->exit = 0;
-	
+
+	CHEROKEE_MUTEX_INIT (&n->active_list_lock, NULL);
+
 	/* Return the object
 	 */
 	*thd = n;
@@ -92,16 +119,24 @@ conn_set_mode (cherokee_thread_t *thd, cherokee_connection_t *conn, cherokee_soc
 static void
 add_connection (cherokee_thread_t *thd, cherokee_connection_t *conn)
 {
+	CHEROKEE_MUTEX_LOCK (&thd->active_list_lock);
+
 	list_add ((list_t *)conn, &thd->active_list);
 	thd->active_list_num++;
+
+	CHEROKEE_MUTEX_UNLOCK (&thd->active_list_lock);
 }
 
 
 static void
 del_connection (cherokee_thread_t *thd, cherokee_connection_t *conn)
 {
+	CHEROKEE_MUTEX_LOCK (&thd->active_list_lock);
+
 	list_del ((list_t *)conn);
 	thd->active_list_num--;
+
+	CHEROKEE_MUTEX_UNLOCK (&thd->active_list_lock);
 }
 
 
@@ -153,7 +188,7 @@ maybe_purge_closed_connection (cherokee_thread_t *thread, cherokee_connection_t
 }
 
 
-static ret_t 
+ret_t 
 process_active_connections (cherokee_thread_t *thd)
 {
 	ret_t ret;
@@ -180,7 +215,7 @@ process_active_connections (cherokee_thread_t *thd)
 		ret = cherokee_fdpoll_check (thd->fdpoll, 
 					     SOCKET_FD(conn->socket), 
 					     SOCKET_STATUS(conn->socket));
-
+		
 		switch (ret) {
 		case -1:
 			purge_closed_connection(thd, conn);
@@ -196,6 +231,19 @@ process_active_connections (cherokee_thread_t *thd)
 		/* Phases
 		 */
 		switch (conn->phase) {
+		case phase_tls_handshake:
+
+			ret = cherokee_socket_init_tls (CONN_SOCK(conn), conn);
+			switch (ret) {
+			case ret_eagain:
+				continue;
+			case ret_error:
+				purge_closed_connection (thd, conn);
+				break;
+			case ret_ok:
+				conn->phase = phase_reading_header;
+			}
+
 		case phase_reading_header:
 
 			ret = cherokee_connection_recv (conn);
@@ -208,7 +256,6 @@ process_active_connections (cherokee_thread_t *thd)
 			 */
 			ret = cherokee_connection_reading_check (conn);
 			if (ret != ret_ok) {
-//				conn_set_mode (thd, conn, stat_writing);
 				conn_set_mode (thd, conn, socket_writing);
 				conn->vserver = srv->vserver_default;
 				conn->phase = phase_add_headers;
@@ -222,7 +269,6 @@ process_active_connections (cherokee_thread_t *thd)
 				
 				/* Turn the connection in write mode
 				 */
-//				conn_set_mode (thd, conn, stat_writing);
 				conn_set_mode (thd, conn, socket_writing);
 			} else 
 				break;
@@ -273,11 +319,12 @@ process_active_connections (cherokee_thread_t *thd)
 					continue;
 				}
 				
-				if (conn->redirect->len >= 1) {
+/*				if (conn->redirect->len >= 1) {
 					cherokee_connection_send_header (conn);
 					maybe_purge_closed_connection (thd, conn);
 					continue;
 				}
+*/
 			} else {
 				ret = cherokee_connection_get_plugin_entry (conn, CONN_VSRV(conn)->plugins, &plugin_entry);
 				if (ret != ret_ok) {
@@ -290,6 +337,14 @@ process_active_connections (cherokee_thread_t *thd)
 				conn->phase = phase_add_headers;
 				continue;
 			}
+
+			/* Check for authentication
+			 */
+			ret = cherokee_connection_check_authentication (conn, plugin_entry);
+			if (ret != ret_ok) {
+				conn->phase = phase_add_headers;
+				continue;
+			}
 			
 			ret = cherokee_connection_create_handler (conn, plugin_entry);
 			if (ret != ret_ok) {
@@ -453,6 +508,8 @@ cherokee_thread_free (cherokee_thread_t *thd)
 		cherokee_connection_free (CONN(i));
 	}
 
+	CHEROKEE_MUTEX_DESTROY (&thd->active_list_lock);
+
 	free (thd);
 
 	return ret_ok;
@@ -490,7 +547,7 @@ cherokee_thread_get_new_connection (cherokee_thread_t *thd, cherokee_connection_
 		new_connection->server = server;
 	} else {
 		new_connection = CONN(thd->reuse_list.next);
-		list_del ((list_t *)new_connection);		
+		list_del ((list_t *)new_connection);	
 		new_connection->timeout = server->bogo_now + server->timeout;
 	}
 
