@@ -34,24 +34,30 @@
 
 #include <pthread.h>
 
+
 typedef struct {
 	struct list_head base;
 	pthread_t        thread;
 
 	cherokee_fdpoll_t *fdpoll;
 
-	void              *server;
-	int                exit;
+	void   *server;
+	int     exit;
 
-	int              active_list_num;
-	list_t           active_list;
-	list_t           reuse_list;
+	int     active_list_num;
+	list_t  active_list;
+	list_t  reuse_list;
+
+#ifdef HAVE_PTHREAD
+        pthread_mutex_t active_list_lock;
+#endif
 
 } cherokee_thread_t;
 
 #define THREAD(x)     ((cherokee_thread_t *)(x))
 #define THREAD_SRV(t) (SRV(THREAD(t)->server))
 
+
 ret_t cherokee_thread_new  (cherokee_thread_t **thd, void *server, int real_thread);
 ret_t cherokee_thread_free (cherokee_thread_t  *thd);
 ret_t cherokee_thread_step (cherokee_thread_t  *thd);
