@@ -23,6 +23,10 @@
  * USA
  */
 
+#include "common.h"
+
+#include <errno.h>
+
 #include "virtual_server.h"
 
 
@@ -39,9 +43,10 @@ cherokee_virtual_server_new (cherokee_virtual_server_t **vserver)
 	vsrv->logger       = NULL;
 	vsrv->logger_props = NULL;
 
-#ifdef HAVE_LIBGNUTLS
+#ifdef HAVE_TLS
 	vsrv->server_cert = NULL;
 	vsrv->server_key  = NULL;
+	vsrv->ca_cert     = NULL;
 #endif
 
 	for (i=0; i<=http_type_400_max; i++) {
@@ -49,10 +54,6 @@ cherokee_virtual_server_new (cherokee_virtual_server_t **vserver)
 		vsrv->error_document400[i].url  = NULL;		
 	}
 	
-	/* Initialize GNU/TLS library if present
-	 */
-	init_gnutls(vsrv);
-
 	cherokee_handler_table_new (&vsrv->plugins);
 	return_if_fail (vsrv->plugins!=NULL, ret_nomem);	   
 
@@ -165,106 +166,6 @@ cherokee_virtual_server_set (cherokee_virtual_server_t *vserver, char *web_dir,
 }
 
 
-static ret_t
-init_gnutls (cherokee_virtual_server_t *vsrv)
-{
-#ifdef HAVE_LIBGNUTLS
-
-	int r;
-	int need_dh_params  = 0;
-	gnutls_dh_params _dh_params[2];
-
-	int need_rsa_params = 0;
-	gnutls_rsa_params _rsa_params[2];
-		
-
-	if ((vsrv->server_cert == NULL) ||
-	    (vsrv->server_key  == NULL))
-	{
-		return ret_error;
-	}
-	
-	r = gnutls_global_init();
-	if (r < 0) {
-		PRINT_ERROR ("gnutls_global_init() = %d\n", r);
-		return ret_error;
-	}
-
-	r = gnutls_global_init_extra();
-	if (r < 0) {
-		PRINT_ERROR ("gnutls_global_init_extra() = %d\n", r);
-		return ret_error;
-	}
-
-	r = gnutls_certificate_allocate_credentials (&vsrv->credentials[0]);
-	if (r < 0) {
-		PRINT_ERROR ("gnutls_certificate_allocate_credentials (&vsrv->credentials[0]=%p) = %d\n", &vsrv->credentials[0], r);
-		return ret_error;
-	}
-
-	r = gnutls_certificate_set_x509_key_file (vsrv->credentials[0],
-						  vsrv->server_cert,
-						  vsrv->server_key,
-						  GNUTLS_X509_FMT_PEM);
-	if (r < 0) {
-		PRINT_ERROR ("gnutls_certificate_set_x509_key_file ('%s', '%s', GNUTLS_X509_FMT_PEM) = %d\n",  
-			     vsrv->server_cert, vsrv->server_key, r);
-		return ret_error;
-	}
-
-	if (vsrv->ca_cert != NULL) {
-		r = gnutls_certificate_set_x509_trust_file (vsrv->credentials[0],
-							    vsrv->ca_cert,
-							    GNUTLS_X509_FMT_PEM);
-		if (r < 0) {
-			PRINT_ERROR ("gnutls_certificate_set_x509_trust_file (..) = %d\n",  r);
-			return ret_error;
-		}
-	}
-	
-	/* Add ciphers 
-	 */
-	vsrv->cipher_priority[0] = GNUTLS_CIPHER_RIJNDAEL_128_CBC;
-	vsrv->cipher_priority[1] = GNUTLS_CIPHER_ARCFOUR_128;
-	vsrv->cipher_priority[2] = GNUTLS_CIPHER_3DES_CBC;
-	vsrv->cipher_priority[3] = GNUTLS_CIPHER_ARCFOUR_40;
-	vsrv->cipher_priority[4] = 0;
-
-	/* Add key exchange methods
-	 */
-	vsrv->kx_priority[0] = GNUTLS_KX_RSA;
-	vsrv->kx_priority[1] = GNUTLS_KX_RSA_EXPORT;
-	vsrv->kx_priority[2] = GNUTLS_KX_DHE_RSA;
-	vsrv->kx_priority[3] = GNUTLS_KX_DHE_DSS;
-	vsrv->kx_priority[4] = 0;
-
-	/* Add MAC Algorithms
-	 */
-	vsrv->mac_priority[0] = GNUTLS_MAC_MD5;
-	vsrv->mac_priority[1] = GNUTLS_MAC_SHA;
-	vsrv->mac_priority[2] = 0;
-
-	/* Add Compression algorithms
-	 */
-	vsrv->comp_priority[0] = GNUTLS_COMP_NULL;
-	vsrv->comp_priority[1] = GNUTLS_COMP_ZLIB;
-	vsrv->comp_priority[2] = GNUTLS_COMP_LZO;
-	vsrv->comp_priority[3] = 0;
-
-	/* Add protocols
-	 */
-	vsrv->protocol_priority[0] = GNUTLS_TLS1;
-	vsrv->protocol_priority[0] = GNUTLS_SSL3;
-	vsrv->protocol_priority[0] = 0;
-
-	
-
-#endif /* HAVE_LIBGNUTLS */
-
-	return ret_ok;
-}
-
-
 ret_t 
 cherokee_virtual_server_get_error_page (cherokee_virtual_server_t *vserver, cherokee_http_t error_code, char **filename_ref)
 {
@@ -295,3 +196,120 @@ cherokee_virtual_server_get_error_url (cherokee_virtual_server_t *vserver, chero
 	
 	return ret_not_found;
 }
+
+
+#ifdef HAVE_TLS
+static ret_t
+generate_dh_params (gnutls_dh_params *dh_params)
+{
+	int          rc;
+	gnutls_datum prime;
+	gnutls_datum generator;
+
+	gnutls_dh_params_init (dh_params);
+	
+	rc = gnutls_dh_params_generate (&prime, &generator, 1024);
+	if (rc < 0) {
+		PRINT_ERROR ("Init TLS: Error in prime generation\n");
+		return ret_error;
+	}
+
+	rc = gnutls_dh_params_set (*dh_params, prime, generator, 1024);
+	if (rc < 0) {
+		PRINT_ERROR ("Init TLS: Error in prime replacement\n");
+		return ret_error;
+	}
+	
+	free (prime.data);
+	free (generator.data);
+
+	return ret_ok;
+}
+
+
+static ret_t
+generate_rsa_params (gnutls_rsa_params* rsa_params)
+{
+	gnutls_datum m, e, d, p, q, u;
+	
+	if (gnutls_rsa_params_init( rsa_params) < 0) {
+		PRINT_ERROR ("tls: Error in rsa parameter initialization\n");
+		return ret_error;
+	}
+
+	/* Generate RSA parameters - for use with RSA-export
+	 * cipher suites. These should be discarded and regenerated
+	 * once a day, once every 500 transactions etc. Depends on the
+	 * security requirements.
+	 */
+
+	if (gnutls_rsa_params_generate(&m, &e, &d, &p, &q, &u, 512) < 0) {
+		PRINT_ERROR ("tls: Error in rsa parameter generation\n");
+		return ret_error;
+	}
+	
+	if (gnutls_rsa_params_set( *rsa_params, m, e, d, p, q, u, 512) < 0) {
+		PRINT_ERROR ("tls: Error in rsa parameter setting\n");
+		return ret_error;
+	}
+	
+	free(m.data);
+	free(e.data);
+	free(d.data);
+	free(p.data);
+	free(q.data);
+	free(u.data);
+	
+	return ret_ok;
+}
+
+
+ret_t 
+cherokee_virtual_server_init_tls (cherokee_virtual_server_t *vsrv)
+{
+	int rc;
+
+	if ((vsrv->ca_cert     == NULL) ||
+	    (vsrv->server_cert == NULL) ||
+	    (vsrv->server_key  == NULL)) 
+	{
+		return ret_error;
+	}
+
+        rc = gnutls_certificate_allocate_credentials (&vsrv->credentials);
+	if (rc < 0) {
+		PRINT_ERROR ("Couldn't allocate credentials.\n");
+		return ret_error;
+	}
+	
+	/* NOTE:
+	 * Hardcoded to use X.509 certificates
+	 */
+	rc = gnutls_certificate_set_x509_trust_file (vsrv->credentials,
+						     vsrv->ca_cert,
+						     GNUTLS_X509_FMT_PEM);
+	if (rc < 0) {
+		PRINT_ERROR ("Error reading X.509 CA Certificate: '%s'\n", vsrv->ca_cert);
+		return ret_error;
+	}
+
+
+	rc = gnutls_certificate_set_x509_key_file (vsrv->credentials,
+						   vsrv->server_cert,
+						   vsrv->server_key,
+						   GNUTLS_X509_FMT_PEM);	
+	if (rc < 0) {
+		PRINT_ERROR ("Error reading X.509 key '%s' or certificate '%s' file\n", 
+			     vsrv->server_key, vsrv->server_cert);
+		return ret_error;
+	}
+
+	generate_dh_params  (&vsrv->dh_params);
+	generate_rsa_params (&vsrv->rsa_params);
+
+	gnutls_certificate_set_rsa_params (vsrv->credentials, vsrv->rsa_params);
+	gnutls_certificate_set_dh_params (vsrv->credentials, vsrv->dh_params);
+
+	return ret_ok;
+}
+#endif /* HAVE_TLS */
