@@ -28,9 +28,9 @@
 
 #include "common.h"
 
-#ifdef HAVE_LIBGNUTLS
-#include <gnutls/extra.h>
-#include <gnutls/gnutls.h>
+#ifdef HAVE_TLS
+# include <gnutls/extra.h>
+# include <gnutls/gnutls.h>
 #endif
 
 #include "handler_table.h"
@@ -53,19 +53,14 @@ typedef struct {
 		char *url;
 	} error_document400[http_type_400_max+1];
 
-
-#ifdef HAVE_LIBGNUTLS
+#ifdef HAVE_TLS
 	char *server_cert;
 	char *server_key;
 	char *ca_cert;
 
-	int   kx_priority[16];
-	int   mac_priority[16];
-	int   comp_priority[16];
-	int   cipher_priority[16];
-	int   protocol_priority[16];
-
-	gnutls_certificate_server_credentials credentials[2];
+	gnutls_certificate_server_credentials credentials;
+	gnutls_dh_params                      dh_params;
+	gnutls_rsa_params                     rsa_params;
 #endif
 
 } cherokee_virtual_server_t;
@@ -77,7 +72,8 @@ ret_t cherokee_virtual_server_new   (cherokee_virtual_server_t **vserver);
 ret_t cherokee_virtual_server_free  (cherokee_virtual_server_t  *vserver);
 ret_t cherokee_virtual_server_clean (cherokee_virtual_server_t  *vserver);
 
-ret_t cherokee_virtual_server_set   (cherokee_virtual_server_t  *vserver, char *web_dir, char *handler_name, cherokee_handler_table_entry_t *plugin_entry);
+ret_t cherokee_virtual_server_init_tls (cherokee_virtual_server_t *vserver);
+ret_t cherokee_virtual_server_set      (cherokee_virtual_server_t *vserver, char *web_dir, char *handler_name, cherokee_handler_table_entry_t *plugin_entry);
 
 ret_t cherokee_virtual_server_get_error_page (cherokee_virtual_server_t *vserver, cherokee_http_t error_code, char **filename_ref);
 ret_t cherokee_virtual_server_get_error_url  (cherokee_virtual_server_t *vserver, cherokee_http_t error_code, char **url_ref);
